package DiffPrinter::html;

use strict;

sub new {
  my $class = shift;
  $class = ref($class) || $class;
  my $this = {};
  bless $this, $class;

  $this->{COLLAPSED} = $_[0];
  $this->{BONSAI_URL} = $_[1];
  $this->{LXR_URL} = $_[2];
  $this->{LXR_ROOT} = $_[3];

  return $this;
}

sub start_patch {
}

sub end_patch {
}

sub start_file {
  my $this = shift;
  my ($file) = @_;

  if ($file->{canonical} && $file->{old_revision} && $this->{BONSAI_URL}) {
    $this->{BONSAI_PREFIX} = "$this->{BONSAI_URL}/cvsblame.cgi?file=$file->{filename}&rev=$file->{old_revision}";
  }
  if ($file->{canonical} && $this->{LXR_URL}) {
    # Cut off the lxr root, if any
    my $filename = $file->{filename};
    $filename = substr($filename, length($this->{LXR_ROOT}));
    $this->{LXR_PREFIX} = "$this->{LXR_URL}/source/$filename";
  }

  my $twisty = $this->{COLLAPSED} ? "(+)" : "(-)";
  my $file_class = $this->{COLLAPSED} ? "file_collapse" : "file";
  my $initial_checked = $this->{COLLAPSED} ? "" : " checked";
  my $filename = $file->{filename} ? $file->{filename} : "";
  print '<div class="file_head"><a href="#" onclick="return twisty_click(this)">',
        $twisty, '</a><input type="checkbox" name="', $filename, '"',
        $initial_checked, ' style="display: none"> ';
  if ($file->{canonical} && $this->{LXR_URL}) {
    print "<a href=\"$this->{LXR_PREFIX}\">$filename</a>";
  } else {
    print $filename;
  }
  print "</div>";
  print "<div class=\"$file_class\">\n";
  print "<script>incremental_restore()</script>\n";
}

sub end_file {
  print "</div>\n";
}

sub section {
  my $this = shift;
  my ($section) = @_;

  my $old_end = $section->{old_lines} ?
                $section->{old_start} + $section->{old_lines} - 1 : 0;
  print '<div class="section_head">Line ';
  if ($this->{BONSAI_PREFIX}) {
    print "<a href=\"$this->{BONSAI_PREFIX}#$section->{old_start}\">";
  }
  if ($this->{BONSAI_PREFIX}) {
    print "</a>";
  }
  print "</div>\n";
  print "<div class=\"section\">\n";

  # Get groups of lines and print them
  my $last_line_char = '';
  my @context_lines;
  my @plus_lines;
  my @minus_lines;
  foreach my $line (@{$section->{lines}}) {
    $line =~ s/\r?\n?$//;
    if ($line =~ /^ /) {
      if ($last_line_char ne ' ') {
        print_lines(\@context_lines, \@plus_lines, \@minus_lines);
        $last_line_char = '';
      }
      $last_line_char = ' ';
      push @context_lines, substr($line, 1);
    } elsif ($line =~ /^\+/) {
      if ($last_line_char eq ' ' || $last_line_char eq '-' && @plus_lines) {
        print_lines(\@context_lines, \@plus_lines, \@minus_lines);
        $last_line_char = '';
      }
      $last_line_char = '+';
      push @plus_lines, substr($line, 1);
    } elsif ($line =~ /^-/) {
      if ($last_line_char eq '+' && @minus_lines) {
        print_lines(\@context_lines, \@plus_lines, \@minus_lines);
        $last_line_char = '';
      }
      $last_line_char = '-';
      push @minus_lines, substr($line, 1);
    }
  }

  print_lines(\@context_lines, \@plus_lines, \@minus_lines);
  print "</div>\n";
}

sub print_lines {
  my ($context_lines, $plus_lines, $minus_lines) = @_;
  if (@{$context_lines}) {
    print "<table>\n";
    foreach my $line (@{$context_lines}) {
      print_line(undef, undef, $line, undef, $line);
    }
    print "</table>\n";
    @{$context_lines} = ();
  }
  return if !@{$plus_lines} && !@{$minus_lines};
  if (@{$plus_lines}) {
    if (@{$minus_lines}) {
      # Plus and minus (change)
      print "<table class=\"changed\">\n";
      for (my $i=0; $i < @{$plus_lines} || $i < @{$minus_lines}; $i++) {
        print_line(undef, undef, $minus_lines->[$i], undef, $plus_lines->[$i]);
      }
      print "</table>\n";
    } else {
      # Plus only
      print "<table>\n";
      foreach my $line (@{$plus_lines}) {
        print_line(undef, undef, undef, "added", $line);
      }
      print "</table>\n";
    }
  } else {
    if (@{$minus_lines}) {
      # Minus only
      print "<table>\n";
      foreach my $line (@{$minus_lines}) {
        print_line(undef, "removed", $line, undef, undef);
      }
      print "</table>\n";
    } else {
      # Nothing
      return;
    }
  }
  @{$plus_lines} = ();
  @{$minus_lines} = ();
}

sub print_line {
  my ($mainclass, $class1, $line1, $class2, $line2) = @_;
  print "<tr" . ($mainclass ? " class=\"$mainclass\"" : "") . ">";
  print "<td" . ($class1 ? " class=\"$class1\"" : "") . ">" . ($line1 ? $line1 : "") . "</td>";
  print "<td" . ($class2 ? " class=\"$class2\"" : "") . ">" . ($line2 ? $line2 : "") . "</td>";
  print "</tr>\n";
}

sub head_section {
 return <<EOM;
<style>
.file_head { font-size: x-large; font-weight: bold; background-color: #d3d3d3; border-top: 1px solid black; border-left: 1px solid black; border-right: 1px solid black; width: 100% }
.file_collapse { display: none }
.section_head { width: 100%; font-weight: bold; background-color: #d3d3d3; border: 1px solid black }
.section { font-family: monospace }
div.section table { width: 100%; empty-cells: show; border-spacing: 0px; border-collapse: collapse }
div.section td { width: 50%; height: 1em; font-size: 0.9em }
.changed { background-color: lightblue }
.added { background-color: lightgreen }
.removed { background-color: lightgreen }
</style>
<script>
function collapse_all(expand) {
  var elem = document.checkboxform.firstChild;
  while (elem != null) {
    if (expand) {
      if (elem.className == 'file_collapse') {
        elem.className = 'file';
        elem.previousSibling.firstChild.firstChild.nodeValue = '(-)';
        elem.previousSibling.firstChild.nextSibling.checked = true;
      }
    } else {
      if (elem.className == 'file') {
        elem.className = 'file_collapse';
        elem.previousSibling.firstChild.firstChild.nodeValue = '(+)';
        elem.previousSibling.firstChild.nextSibling.checked = false;
      }
    }
    elem = elem.nextSibling;
  }
  return false;
}

var current_restore_elem;

function restore_all() {
  current_restore_elem = null;
  incremental_restore();
}

function incremental_restore() {
  if (!document.checkboxform.restore_indicator.checked) {
    return;
  }
  var next_restore_elem;
  if (current_restore_elem) {
    next_restore_elem = current_restore_elem.nextSibling;
  } else {
    next_restore_elem = document.checkboxform.firstChild;
  }
  while (next_restore_elem != null) {
    current_restore_elem = next_restore_elem;
    restore_elem(current_restore_elem);
    next_restore_elem = current_restore_elem.nextSibling;
  }
}

function restore_elem(elem, alertme) {
  if (elem.className == 'file_collapse') {
    if (elem.previousSibling.firstChild.nextSibling.checked) {
      elem.className = 'file';
      elem.previousSibling.firstChild.firstChild.nodeValue = '(-)';
    }
  } else if (elem.className == 'file') {
    if (!elem.previousSibling.firstChild.nextSibling.checked) {
      elem.className = 'file_collapse';
      elem.previousSibling.firstChild.firstChild.nodeValue = '(+)';
    }
  }
}

function twisty_click(twisty) {
  if (twisty.parentNode.nextSibling.className == 'file') {
    twisty.parentNode.nextSibling.className = 'file_collapse';
    twisty.firstChild.nodeValue = '(+)';
    twisty.nextSibling.checked = false;
  } else {
    twisty.parentNode.nextSibling.className = 'file';
    twisty.firstChild.nodeValue = '(-)';
    twisty.nextSibling.checked = true;
  }
  return false;
}
</script>
EOM
}

sub onload {
  return "restore_all(); document.checkboxform.restore_indicator.checked = true";
}

1
